@@ -1,6 +1,6 @@
 # -*- coding: iso-8859-1 -*-
 # GNU Solfege - free ear training software
-# Copyright (C) 2001, 2002, 2003, 2004, 2007, 2008  Tom Cato Amundsen
+# Copyright (C) 2001, 2002, 2003, 2004, 2007, 2008, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,18 +22,17 @@ prog             The test done before calling
  +statementlist
   +statement
    +assignment   peek: 'NAME', '='
-    +faktorlist  scan('NAME') scan('=')
-     +faktor
+    +expressionlist  scan('NAME') scan('=')
+     +expression
       +atom()  kalles direkt p frste linje. S evt p nytt etter +-/%
        +functioncall    peek: 'NAME' '('
-        +faktorlist     peek() != ')'
+        +expressionlist     peek() != ')'
    +block        peek: 'NAME', '{'
-    +assignmentlist
-    +faktor     peek_type()!= '}'
+    +assignments
+    +expression     peek_type()!= '}'
    +include      peek: 'NAME'("include"), '(
     +prog
 
-assignmentlist peek: 'NAME' '='
 +assignment
 
 """
@@ -41,17 +40,22 @@ assignmentlist peek: 'NAME' '='
 # P singchord-1 sparer jeg ikke noe p  ha en peek2_type(t1, t2)
 # som tester de to neste token.
 
+import codecs
 import os
 import re
 import sys
 import weakref
 
 from solfege import i18n
+import solfege.parsetree as pt
 
-tokens = ('NAME', 'STRING', 'OPERATOR', 'INTEGER', 'FLOAT', 'CHAR', 'EOF')
-for t in tokens:
-    globals()[t] = t
-del t
+NAME = 'NAME'
+STRING = 'STRING'
+OPERATOR = 'OPERATOR'
+INTEGER = 'INTEGER'
+FLOAT = 'FLOAT'
+CHAR = 'CHAR'
+EOF = 'EOF'
 
 NEW_re = re.compile("""(?:
                         (\s+)|  #space
@@ -95,6 +99,14 @@ class istr(unicode):
     def __init__(self, s):
         self.cval = s
         self.m_added_language = None
+    def __mod__(self, other):
+        """
+        Handle format strings in translated strings:
+        _("%i. inversion") % 2
+        """
+        i=istr(unicode(self) % other)
+        i.cval = self.cval % other
+        return i
     def add_translation(self, lang, s):
         """
         Use this method to add translations that are included directly in
@@ -124,11 +136,15 @@ class istr(unicode):
     new_translated = staticmethod(new_translated)
 
 def dataparser_i18n_func(s):
+    if s == "":
+        s = 'error in your lesson file: empty string cannot be translated'
     retval = istr(_(s))
     retval.cval = s
     return retval
 
 def dataparser_i18n__i_func(s):
+    if s == "":
+        s = 'error in your lesson file: empty string cannot be translated'
     retval = istr(_i(s))
     retval.cval = s
     return retval
@@ -148,28 +164,6 @@ class DataparserException(Exception):
         Exception.__init__(self, message)
 
 
-class NameLookupException(DataparserException):
-    def __init__(self, parser, bad_pos):
-        DataparserException.__init__(self,
-            _("Unknown name \"%(name)s\" in line %(line)i of file \"%(filename)s\":") % {
-                'name': parser._lexer.m_tokens[bad_pos][TOKEN_STRING],
-                'line': parser._lexer.m_tokens[bad_pos][TOKEN_LINENO],
-                'filename': parser.m_filename})
-        # This variable is only used by the module test code.
-        self.m_token = parser._lexer.m_tokens[bad_pos]
-        self.m_nonwrapped_text = parser._lexer.get_err_context(bad_pos)
-
-class WrongArgumentCount(DataparserException):
-    def __init__(self, parser, bad_pos):
-        DataparserException.__init__(self,
-            _("Wrong argument count in line %(line)i of file \"%(filename)s\":") % {
-                'line': parser._lexer.m_tokens[bad_pos][TOKEN_LINENO],
-                'filename': parser.m_filename})
-        # This variable is only used by the module test code.
-        self.m_token = parser._lexer.m_tokens[bad_pos]
-        self.m_nonwrapped_text = parser._lexer.get_err_context(bad_pos)
-
-
 class DataparserSyntaxError(DataparserException):
     def __init__(self, parser, bad_pos, expect):
         DataparserException.__init__(self, _('Syntax error in file "%(filename)s". %(expected)s') % {'filename': parser.m_filename, 'expected': expect})
@@ -184,9 +178,9 @@ class AssignmentToReservedWordException(DataparserException):
         self.m_token = parser._lexer.m_tokens[bad_pos]
         self.m_nonwrapped_text = parser._lexer.get_err_context(bad_pos)
 
-class CannotTranslateListsException(DataparserException):
+class CanOnlyTranslateStringsException(DataparserException):
     def __init__(self, parser, bad_pos, variable):
-        DataparserException.__init__(self, _("Cannot translate lists using in-file translations (ex var[no]=...). See the variable \"%(variable)s\" in the file \"%(filename)s\"") % {'filename': parser.m_filename, 'variable': variable})
+        DataparserException.__init__(self, _("We can only translate strings using in-file translations (ex var[no]=...). See the variable \"%(variable)s\" in the file \"%(filename)s\"") % {'filename': parser.m_filename, 'variable': variable})
         # This variable is only used by the module test code.
         self.m_token = parser._lexer.m_tokens[bad_pos]
         self.m_nonwrapped_text = parser._lexer.get_err_context(bad_pos)
@@ -226,6 +220,9 @@ class Lexer:
         else:
             src = unicode(src, "UTF-8", errors="replace")
         src = src.replace("\r", "\n")
+        # Some editors (notepad on win32?) insert the BOM, so we have
+        # to check for it and remove it since the lexer don't handle it.
+        src = src.lstrip(unicode(codecs.BOM_UTF8, "utf8"))
         self.m_src = src
         self.pos = 0
         pos = 0
@@ -233,8 +230,8 @@ class Lexer:
         self.m_tokens = []
         while 1:
             try:
-                if src[pos] in " \n\t{}=%+,/()":
-                    if src[pos] in ' \t':
+                if src[pos] in u"\u202f\xa0 \n\t{}=%+,/().":
+                    if src[pos] in u'\u202f\xa0 \t':
                         pos += 1
                         continue
                     if src[pos] == '\n':
@@ -282,6 +279,26 @@ class Lexer:
         return self._err_context_worker(len(self.m_tokens)-1)
     def get_err_context(self, pos):
         return self._err_context_worker(pos)
+    def new_get_err_context(self, pos1, pos2):
+        # Line number of the last part of the error. We will display
+        # two lines, the last line containing (part of) the error, and
+        # the line before it.
+        l2 = self.m_src[:pos2].count("\n")
+        # i1 and i2 is the start and end of the text that should be
+        # marked as erroneous. If the error is stretched over several
+        # lines, then we will only mark the last line.
+        i = pos2
+        while i > 0 and self.m_src[i] != "\n":
+            i -= 1
+        i2 = pos2 - i - 1
+        if pos1 > i:
+            i1 = pos1 - i - 1
+        else:
+            i1 = 0
+        linestr = "(line %i):" % (l2 + 1)
+        return ("(line %i): %s\n" % (l2 , self.get_line(l2-1))
+              + "%s %s\n" % (linestr, self.get_line(l2))
+              + " " * (i1 + len(linestr) + 1) + "^" * (i2 - i1))
     def peek(self, forward=0):
         return self.m_tokens[self.pos+forward]
     def peek_type(self, forward=0):
@@ -319,30 +336,20 @@ class Lexer:
             x += 1
         return self.m_src[idx:x]
 
-
 class Dataparser:
-    def __init__(self, globals={}, function_dict={}, gd=[]):
-        self.gd = gd
-        self.globals = globals.copy()
-        self.functions = function_dict.copy()
-        self.header = {}
-        self.questions = []
-        # Each block type will have a list in blocklists,
-        # for example self.blocklists['element'] = []
-        self.blocklists = {}
-        self.context = self.globals
+    """
+    Parse a lesson file into a parsetree.Program
+    """
+    def __init__(self):
         self.m_filename = None
-        self.m_ignore_lookup_error = False
         self.m_translation_re = re.compile("(?P<varname>\w+)\[(?P<lang>[\w_+]+)\]")
     def parse_file(self, filename):
         """We always construct a new parser if we want to parse another
         file. So this method is never called twice for one parser.
         """
         self.m_filename = filename
-        # Lexer will decode the string read
-        infile = open(filename, 'rU')
-        self._lexer = Lexer(infile.read(), self)
-        infile.close()
+        self.m_location = os.path.split(filename)[0]
+        self._lexer = Lexer(open(filename, 'rU').read(), self)
         self.reserved_words = ('_', 'question', 'header')
         self.prog()
     def parse_string(self, s, really_filename=False):
@@ -355,6 +362,9 @@ class Dataparser:
         self.prog()
     def prog(self):
         """prog: statementlist EOF"""
+        self.tree = pt.Program()
+        self.tree.m_filename = self.m_filename
+        self.tree._lexer = self._lexer
         self.statementlist()
         if self._lexer.peek_type() != 'EOF':
             # This exception will be raised if we for example have
@@ -365,29 +375,38 @@ class Dataparser:
     def statementlist(self):
         """statementlist: (statement+)"""
         while self._lexer.peek_type() == 'NAME':
-            self.statement()
-    def statement(self):
-        """statement: assignment | block | include"""
+            self.tree.add_statement(self.statement())
+    def statement(self, allow_music_shortcut=False):
+        """
+        statement: assignment | block | include | import
+        allow_music_shortcut can be set when we are parsing assignments
+        in a question block.
+        """
         if self._lexer.peek_type(1) == '=':
-            self.assignment()
+            return self.assignment()
         elif self._lexer.peek_type(1) == '{':
-            self.block()
+            return self.block()
         elif self._lexer.peek_type(1) == 'NAME' \
                 and self._lexer.peek_type(2) == '{':
-            self.named_block()
+            return self.named_block()
         elif self._lexer.peek_type() == 'NAME' \
                 and self._lexer.peek_string() == 'include' \
                 and self._lexer.peek_type(1) == '(':
-            self.include()
-        else:
-            if self._lexer.peek_type(1) == 'EOF':
-                extra = " Found End of File."
+            return self.include()
+        elif (self._lexer.peek_type() == 'NAME'
+              and self._lexer.peek_string() == 'import'):
+            return self.do_import()
+        elif (self._lexer.peek_type() == 'NAME'
+              and self._lexer.peek_string() == 'rimport'):
+            return self.do_rimport()
+        elif allow_music_shortcut:
+            fakt = self.expressionlist()
+            if self._lexer.peek_type() == '}':
+                return pt.Assignment(pt.Identifier(u"music"), fakt[0])
             else:
-                extra = ""
-            # Add a single A to the end of a valid file to raise
-            # this exception.
-            raise DataparserSyntaxError(self, self._lexer.pos + 1,
-              "Expected token '=' or '{'. %s" % extra)
+                raise DataparserSyntaxError(self, self._lexer.pos, "The (obsolete) music shortcut construct must be the last statement in a question block.")
+        self._lexer.scan_any()
+        raise DataparserSyntaxError(self, self._lexer.pos, "Parse error")
     def include(self):
         self._lexer.scan_any() # scan include
         self._lexer.scan_any() # scan (
@@ -397,204 +416,158 @@ class Dataparser:
             print >> sys.stderr, "Warning: The file '%s' uses old style syntax for the include command." % self.m_filename
             print >> sys.stderr, 'This is not fatal now but will be in the future. You should change the code\nfrom include(filename) to include("filename")\n'
             filename = self._lexer.scan('NAME')
-        old_lexer = self._lexer
-        # don't let the new file pollute my header!
-        old_header = self.header
-        self.header = {}
-        # Lexer will decode the string read
-        ifile = open(os.path.join(self.m_location, filename), 'rU')
-        self._lexer = Lexer(ifile.read(), self)
-        ifile.close()
-        self.prog()
-        self._lexer = old_lexer
-        for k, v in old_header.items():
-            self.header[k] = v
+        fn = os.path.join(self.m_location, filename)
+        if not os.path.exists(fn):
+            fn = os.path.join(os.getcwdu(), u'exercises/standard/lesson-files', filename)
+        s = open(fn, 'rU').read()
+        p = Dataparser()
+        p.parse_string(s)
         self._lexer.scan(')')
-    def assignmentlist(self):
-        """assignmentlist: (assignment+) """
-        # FIXME peek(1) is added because of the music shortcut
-        while self._lexer.peek_type() == 'NAME' and self._lexer.peek_type(1) == '=':
-            self.assignment()
+        return pt.IncludeStatement(p.tree)
+    def _import_worker(self, fn1, fn2):
+        self._lexer.scan_any() # scan the 'import' or 'rimport' keyword
+        mod_filename = self._lexer.scan_any()
+        if (self._lexer.peek_type() == 'NAME'
+            and self._lexer.peek_string() == 'as'):
+            self._lexer.scan_any()
+            mod_name = self._lexer.scan('NAME')
+        else:
+            mod_name = mod_filename
+        p = Dataparser()
+        fn1 = os.path.join(fn1, mod_filename)
+        fn2 = os.path.join(fn2, mod_filename)
+        if os.path.exists(fn1) or not os.path.exists(fn2):
+            p.parse_file(fn1)
+        else:
+            p.parse_file(fn2)
+        return pt.Assignment(pt.Identifier(mod_name), p.tree)
+    def do_import(self):
+        return self._import_worker(
+            os.path.join(os.getcwdu(), "exercises", "standard", "lib"),
+            os.path.join(self.m_location, "..", "lib"))
+    def do_rimport(self):
+        return self._import_worker(
+            os.path.join(self.m_location, "..", "lib"),
+            os.path.join(os.getcwdu(), "exercises", "standard", "lib"))
     def assignment(self):
-        """NAME "=" faktor ("," faktor)* """
+        """NAME "=" expression ("," expression)* """
         npos = self._lexer.pos
         name = self._lexer.scan_any()#('NAME')
         if name in self.reserved_words:
             # do "question = 1" to raise this exception.
             raise AssignmentToReservedWordException(self, npos, name)
         self._lexer.scan_any()#('=')
-        faktorlist = self.faktorlist()
+        fpos = self._lexer.pos
+        expressionlist = self.expressionlist()
         m = self.m_translation_re.match(name)
         if m:
-            if len(faktorlist) != 1:
-                raise CannotTranslateListsException(self, npos, name)
-            faktor = faktorlist[0]
-            assert type(faktor) == istr
-            if m.group('varname') in self.context:
-                self.context[m.group('varname')] = self.context[m.group('varname')].add_translation(m.group('lang'), faktor)
-            else:
-                # add the first translation as cval until we get the correct
-                # value to use.
-                self.context[m.group('varname')] = faktor
-                # Also add faktor as a translation, since it might be the
-                # translation we need.
-                self.context[m.group('varname')] = self.context[m.group('varname')].add_translation(m.group('lang'), faktor)
+            if len(expressionlist) != 1:
+                raise CanOnlyTranslateStringsException(self, fpos, name)
+            if not isinstance(expressionlist[0].m_value, istr):
+                raise CanOnlyTranslateStringsException(self, fpos, name)
+        if len(expressionlist) == 1:
+            return pt.Assignment(pt.Identifier(name), expressionlist[0])
         else:
-            # We only check for cval if len(faktorlist) == 1, because
-            # lists are not localized.
-            if len(faktorlist) == 1:
-                if name in self.context and isinstance(self.context[name], istr):
-                    self.context[name].cval = faktorlist[0]
-                else:
-                    self.context[name] = faktorlist[0]
-            else:
-                self.context[name] = faktorlist
-    def faktor(self):
-        """faktor: atom
+            return pt.Assignment(pt.Identifier(name), expressionlist)
+    def expression(self):
+        """expression: atom
               ("+" atom
               |"-" atom
               |"/" atom
               )*
               """
-        faktor = self.atom()
+        # use tmp var just in case an atom in the future is more than
+        # one token.
+        n = self._lexer.pos
+        expression = self.atom()
+        expression.m_tokenpos = n
         peek = self._lexer.peek_type()
         while 1:
             if peek == '+':
                 self._lexer.scan_any()
-                faktor += self.atom()
+                expression = pt.Addition(expression, self.atom())
             elif peek == '-':
                 self._lexer.scan_any()
-                faktor -= self.atom()
+                expression -= self.atom()
             elif peek == '/':
                 self._lexer.scan_any()
-                faktor = (faktor, self.atom())
+                expression = pt.TempoType(expression, self.atom())
             elif peek == '%':
                 self._lexer.scan_any()
-                faktor = faktor % self.atom()
+                expression = pt.StringFormatting(expression, self.atom())
             else:
                 break
             peek = self._lexer.peek_type()
-        return faktor
-    def faktorlist(self):
-        """faktorlist: faktor ("," faktor)* """
-        faktorlist = [self.faktor()]
+        return expression
+    def expressionlist(self):
+        """expressionlist: expression ("," expression)* """
+        # use tmp var just in case an atom in the future is more than
+        # one token.
+        n = self._lexer.pos
+        expressionlist = pt.ExpressionList([self.expression()])
+        expressionlist.m_tokenpos = n
         while self._lexer.peek_type() == ',':
             self._lexer.scan_any()
-            faktorlist.append(self.faktor())
-        return faktorlist
+            expressionlist.append(self.expression())
+        return expressionlist
     def atom(self):
         """atom: INTEGER | FLOAT | STRING | NAME | FUNCTIONCALL"""
         npos = self._lexer.pos
         peek = self._lexer.peek_type()
         if peek == 'STRING':
-            return istr(self._lexer.scan('STRING'))
+            return pt.Literal(istr(self._lexer.scan('STRING')))
         elif peek == 'INTEGER':
-            return int(self._lexer.scan('INTEGER'))
+            return pt.Literal(int(self._lexer.scan('INTEGER')))
         elif peek == 'FLOAT':
-            return float(self._lexer.scan('FLOAT'))
+            return pt.Literal(float(self._lexer.scan('FLOAT')))
         elif peek == 'NAME':
             if self._lexer.peek_type(1) == '(':
                 return self.functioncall()
-            try:
-                return self.lookup_name(self._lexer.scan('NAME'))
-            except KeyError:
-                # Tested in TestDataParser.test_exception_atom
-                raise NameLookupException(self, npos)
+            name = self._lexer.scan('NAME')
+            while self._lexer.peek_type() == '.':
+                self._lexer.scan_any()
+                if self._lexer.peek_type() != 'NAME':
+                    raise DataparserSyntaxError(self, self._lexer.pos, 'Identifier expected')
+                name += "." + self._lexer.scan('NAME')
+            return pt.Identifier(name)
         else:
             #print "FIXME: have no idea how to raise this exception"
             raise DataparserSyntaxError(self, npos + 1,
                 "Expected STRING, INTEGER or NAME+'('")
     def functioncall(self):
-        """functioncall: NAME "(" faktorlist ")" """
+        """functioncall: NAME "(" expressionlist ")" """
         npos = self._lexer.pos
         name = self._lexer.scan_any()#'NAME')
         self._lexer.scan('(')
         if self._lexer.peek_type() == ')':
             # functioncall()
+            ret = pt.FunctionCall(self, name, [])
             self._lexer.scan(')')
-            try:
-                if self.functions[name][0]:
-                    return self.functions[name][1](self)
-                else:
-                    return self.functions[name][1]()
-            except KeyError:
-                raise NameLookupException(self, npos)
         else:
             # functioncall(arglist)
-            arglist = self.faktorlist()
+            arglist = self.expressionlist()
             self._lexer.scan(')')
-            try:
-                if self.functions[name][0]:
-                    return self.functions[name][1](self, *arglist)
-                else:
-                    return self.functions[name][1](*arglist)
-            except KeyError:
-                raise NameLookupException(self, npos)
-            except TypeError:
-                raise WrongArgumentCount(self, npos)
+            ret = pt.FunctionCall(self, name, arglist)
+        ret.m_tokenpos = npos
+        return ret
     def block(self):
-        """block: NAME "{" assignmentlist "}" """
-        name = self._lexer.scan_any()
-        if name == 'header':
-            self.context = self.header
-        elif name == 'question':
-            self.questions.append(Question())
-            self.context = self.questions[-1]
-        else:
-            if name not in self.blocklists:
-                self.blocklists[name] = []
-            self.blocklists[name].append(dict())
-            self.context = self.blocklists[name][-1]
+        """block: NAME "{" assignments"}" """
+        block = pt.Block(self._lexer.scan_any())
         self._lexer.scan_any() # scan '{'
-        # The question block is a little more code because of the shortcut
-        # we allow: question { "music string }
-        if name == 'question':
-            self.assignmentlist()
-            if self._lexer.peek_type() != '}':
-                self.context['music'] = self.faktor()
-        # The single line two below is the code needed if we dont' have
-        # shortcuts. Currently the headerblock goes here.
-        else:
-            self.assignmentlist()
+        # FIXME this is not nice, but we need it since we have
+        # allowed question blocks with the shortcut where they
+        # omit "music ="
+        while self._lexer.peek_type() != '}':
+            block.add_statement(self.statement(allow_music_shortcut=True))
         self._lexer.scan("}")
-        if name == 'question': #FIXME this is code I want to remove.
-            for n in self.gd:
-                if not (n in self.context):
-                    self.context[n] = self.globals[n]
-        self.context = self.globals
+        return block
     def named_block(self):
         blocktype = self._lexer.scan('NAME')
         name = self._lexer.scan('NAME')
-        #FIXME right now named_block is reserved to element blocks, but
-        # I hope to move other blocks here too. Or at least questions should
-        # use self.blocklists, I think.
-        if blocktype != 'element':
-            raise DataparserSyntaxError(self, self._lexer.pos - 2, 'The only named block allowed are "element"')
-        if blocktype not in self.blocklists:
-            self.blocklists[blocktype] = []
-        elem = dict()
-        # We must add the name of the block to the global name space since
-        # it will be referred from other blocks.
-        self.globals[name] = elem
-        # And they have to be added to the list of blocks because we may
-        # need to access all blocks of a certain type.
-        self.blocklists[blocktype].append(elem)
-        elem['name'] = name
+        block = pt.NamedBlock(blocktype, name)
         self._lexer.scan('{')
-        self.context = elem
-        self.assignmentlist()
+        while self._lexer.peek_type() == 'NAME':
+            block.add_statement(self.statement())
         self._lexer.scan("}")
-        self.context = self.globals
-    def lookup_name(self, name):
-        """
-        Raises KeyError if the name is not found.
-        """
-        if name in self.context:
-            return self.context[name]
-        elif name in self.globals:
-            return self.globals[name]
-        else:
-            if self.m_ignore_lookup_error:
-                return "LOOKUP IGNORED"
-            raise KeyError
+        return block
 
