@@ -21,6 +21,7 @@ import locale
 import logging
 import sys
 
+import gobject
 import gtk
 
 import solfege
@@ -172,9 +173,13 @@ class ExerciseView(SelectWinBase):
         do_sg = True
         if len(page[0]) > 0 and isinstance(page[0][0], frontpage.LinkList):
             count = 0
-            for p in page[0]:
-                if len(p):
-                    count += len(p[0])
+            for column in page:
+                assert isinstance(column, frontpage.Column)
+                for linklist in column:
+                    assert isinstance(linklist, (frontpage.LinkList,
+                                                 frontpage.Paragraph))
+                    if isinstance(linklist, frontpage.LinkList):
+                        count += len(linklist)
             if count > 80:
                 do_sg = False
         for column in page:
@@ -187,6 +192,7 @@ class ExerciseView(SelectWinBase):
             for sect_id, linklist in enumerate(column):
                 if isinstance(linklist, frontpage.Paragraph):
                     label = gtk.Label(linklist.m_text)
+                    label.set_use_markup(True)
                     label.set_line_wrap(True)
                     label.set_alignment(0.0, 0.5)
                     vbox.pack_start(label, False)
@@ -219,7 +225,8 @@ class ExerciseView(SelectWinBase):
                                 if fieldname in (u'link', u'link-with-filename-tooltip'):
                                     labeltxt = lessonfile.infocache.get(link, 'title')
                                     lbl = gu.ClickableLabel(_no_xgettext(labeltxt))
-                                    if fieldname == u'link-with-filename-tooltip':
+                                    if (fieldname == u'link-with-filename-tooltip'
+                                        or solfege.app.m_options.debug):
                                         lbl.set_tooltip_text(link)
                                     if do_sg:
                                         sizegroups[fieldname].add_widget(lbl)
@@ -279,8 +286,10 @@ class ExerciseView(SelectWinBase):
         self.get_vadjustment().set_value(0.0)
     def on_page_link_clicked(self, btn, link):
         self.g_searchbox.hide()
+        self.emit('link-clicked', self.m_page)
         self.display_data(link)
     def on_link_clicked(self, w, filename):
+        self.emit('link-clicked', self.m_page)
         solfege.app.practise_lessonfile(filename)
     def display_search_result(self, searchfor, result, result_C, display_only_tests=False):
         self._display_data(
@@ -332,6 +341,11 @@ class ExerciseView(SelectWinBase):
                     pass
         return page
 
+gobject.signal_new('link-clicked', ExerciseView,
+                   gobject.SIGNAL_RUN_FIRST,
+                   gobject.TYPE_NONE,
+                   (gobject.TYPE_PYOBJECT,))
+
 
 class FrontPage(ExerciseView):
     def display_data(self, data, display_only_tests=False,
@@ -353,6 +367,7 @@ class FrontPage(ExerciseView):
 
 class TestsView(ExerciseView):
     def on_link_clicked(self, w, filename):
+        self.emit('link-clicked', self.m_page)
         solfege.app.test_lessonfile(filename)
     def display_data(self, data=None, is_search_result=False, show_topics=False):
         self._display_data(data, True, is_search_result, show_topics)
