@@ -278,6 +278,10 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
                 solfege.splash_win.show_progress(fn)
             if not frontpage.may_be_frontpage(fn):
                 continue
+            try:
+                title = lessonfile.infocache.frontpage.get(fn, 'title')
+            except TypeError:
+                continue
             cur_dir = os.path.split(fn)[0]
             if old_dir != cur_dir:
                 s += '<separator name="sep@%s"/>' % fn
@@ -519,6 +523,9 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
             print >> sys.stderr, "\n".join(textwrap.wrap("Translations are disabled because your locale settings are broken. This is not a bug in GNU Solfege, so don't report it. The README file distributed with the program has some more details."))
         except AttributeError:
             pass
+        for filename in lessonfile.infocache.frontpage.iter_old_format_files():
+            gu.dialog_ok(_("Cannot load front page file"), None,
+                _("The file «%s» is saved in an old file format. The file can be converted by editing and saving it with an older version of Solfege. Versions from 3.16.0 to 3.20.4 should do the job.") % filename)
     def activate_exercise(self, module, urlobj=None):
         self.show_view(module)
         # We need this test because not all exercises use a notebook.
