@@ -81,11 +81,6 @@ class DB(object):
             pass
         self.conn = sqlite3.connect(statistics_filename)
         self.setup_tables()
-        # We don't read old data if we are running another profile than
-        # the default one, since there are no old format statistics saved
-        # in the profiles
-        if self.must_recreate and not profile:
-            self.read_old_data(callback)
         self.upgrade_to_version_2()
     def insert_file(self, filename):
         assert lessonfile.is_uri(filename) or os.path.isabs(filename)
@@ -155,154 +150,6 @@ class DB(object):
             return row[0]
         else:
             raise self.FileNotInDB(filename)
-    def get_uuid_to_filename_mapping(self, callback=None):
-        """
-        Parse all lesson files we can find and return a dict mapping
-        the lesson_id to file names.
-        Current lessonfile manager code cause us to parse the user contributed
-        files in ~/lessonfiles too.
-        """
-        count = 0
-        mapping = {}
-        lessonfile.require_mgr()
-        for uuid, data in lessonfile.mgr.parse(False):
-            mapping[uuid] = lessonfile.mk_uri(data['filename'])
-            count += 1
-            callback(_("Files read: %i") % count)
-        logging.info("get_uuid_to_filename_mapping: scanned %i files.", count)
-        return count, mapping
-    def read_old_data(self, callback):
-        """
-        This function should be run once to import the old format statistics
-        from ~/.solfege/statistics and into the sqlite database.
-        We will import statistics for all uuids that exist in the lessonfiles table.
-        """
-        # Snippet from http://wiki.python.org/moin/UsingPickle/RenamingModules
-        # that let us rename even though we have a renamed class
-        renametable = {
-            'src.dataparser': 'solfege.dataparser',
-            }
-
-        def mapname(name):
-            if name in renametable:
-                return renametable[name]
-            return name
-
-        def mapped_load_global(self):
-            module = mapname(self.readline()[:-1])
-            name = mapname(self.readline()[:-1])
-            klass = self.find_class(module, name)
-            self.append(klass)
-
-        def load(file):
-            unpickler = pickle.Unpickler(file)
-            unpickler.dispatch[pickle.GLOBAL] = mapped_load_global
-            return unpickler.load()
-
-        imp, uuid_to_filename = self.get_uuid_to_filename_mapping(callback)
-        def read_session_data(datadir, is_tests, counter):
-            if not os.path.isdir(datadir):
-                logging.debug("read_session_data: '%s' does not exist. Returning", datadir)
-                return
-            for uuid in os.listdir(datadir):
-                if not (os.path.isdir(os.path.join(datadir, uuid))
-                    and os.path.isfile(os.path.join(datadir, '%s_hash' % uuid))):
-                    continue
-                # We do not add statistics for lesson files we don't have
-                # access to.
-                if not uuid in uuid_to_filename:
-                    continue
-                lessonfilename = uuid_to_filename[uuid]
-                try:
-                    fileid = self.get_fileid(lessonfilename)
-                except self.FileNotInDB:
-                    fileid = self.insert_file(lessonfilename)
-                # saved_hashvalue will get the hash value as solfege 3.14 (or
-                # older) calculated when the statistics was saved.
-                try:
-                    s = open(os.path.join(datadir, u"%s_hash" % uuid), "r").read()
-                    try:
-                        saved_hashvalue = int(s)
-                    except ValueError:
-                        saved_hashvalue = None
-                except IOError:
-                    saved_hashvalue = None
-                real_lessonfilename = lessonfile.uri_expand(lessonfilename)
-                s = open(real_lessonfilename, "r").read()
-                # Check if the lesson file in this version of solfege is equal to the
-                # one used when saving the statistics.
-                if hash(s) != saved_hashvalue:
-                    # header.replaces is not used when importing statistics.
-                    # This because the old statistics saved the hash value
-                    # using python standard hash function, and the function
-                    # does not return the same value on all systems.
-                    def bugfix_copy_fn(filename, subdir):
-                        """
-                        Return the filename of the copy of the file if it
-                        exists. Return filename if not.
-                        """
-                        d, f = os.path.split(filename)
-                        fn = os.path.join(subdir, f)
-                        if (d == 'exercises%sstandard%slesson-files' % (os.sep, os.sep)
-                            and os.path.isfile(fn)):
-                            return fn
-                        return filename
-
-                    def bugfix_hash_file(filename, subdir):
-                        copy_fn = bugfix_copy_fn(filename, subdir)
-                        if os.path.isfile(copy_fn):
-                            return hash(open(copy_fn, 'r').read())
-                        return None
-
-                    if not (saved_hashvalue == bugfix_hash_file(real_lessonfilename, "hash-bug-workaround")
-                            or saved_hashvalue == bugfix_hash_file(real_lessonfilename, "hash-bug-workaround2")):
-                        logging.debug("Ignoring statistics for '%s' since the saved hash value does not match.", lessonfilename)
-                        continue
-
-                for timestamp in os.listdir(os.path.join(datadir, uuid)):
-                    if timestamp == u"passed":
-                        continue
-                    f = open(os.path.join(datadir, uuid, timestamp), 'r')
-                    session = load(f)
-                    f.close()
-                    for correct in session.keys():
-                        for guess in session[correct]:
-                            self.conn.execute("insert into sessions "
-                                "(fileid, timestamp, answerkey, guessed, count) "
-                                "values(?, ?, ?, ?, ?)",
-                            (fileid, timestamp, unicode(correct), unicode(guess), session[correct][guess]))
-                    self.conn.execute("insert into sessioninfo "
-                        "(fileid, timestamp, sessiontype) "
-                        "values(?, ?, ?)",
-                        (fileid, timestamp, int(bool(is_tests))))
-                    counter += 1
-                    callback(_("Files read: %i") % counter)
-                if is_tests:
-                    timestamp = self.conn.execute("select timestamp from sessioninfo "
-                        "where fileid=? and sessiontype=1 "
-                        "order by -timestamp ",
-                        (fileid,)).fetchone()[0]
-                    if timestamp:
-                        if lessonfile.infocache.get(lessonfilename, "module") in ('melodicinterval', 'harmonicinterval', 'singinterval'):
-                            parserclass = lessonfile.IntervalsLessonfile
-                        else:
-                            parserclass = lessonfile.IdByNameLessonfile
-                        p = parserclass()
-                        p.parse_file(lessonfilename)
-                        self.cache_new_test_result(lessonfilename,
-                            timestamp,
-                            p.get_test_requirement(),
-                            p.get_test_num_questions())
-            return imp
-        imp = read_session_data(
-                os.path.join(filesystem.app_data(), 'statistics'),
-                False, imp)
-        imp = read_session_data(
-                os.path.join(filesystem.app_data(), 'testresults'),
-                True, imp)
-        self.set_variable("database_version", 2)
-        logging.info("imported statistics for %s lessonfiles", imp)
-        self.conn.commit()
     def upgrade_to_version_2(self):
         """
         After releasing 3.16.0 I found that having separate tests and
